#ifndef _UFS_FS_H
#define _UFS_FS_H

#include <linux/fs.h>

/* endianness conversions */
/* host <--> disk for "short"s */
static inline unsigned short dtohs (struct ufs_sb_info * sb, unsigned short x)
{
	if (sb->fs_convert) {
#ifdef __i386__
		__asm__("xchgb %b0,%h0" : "=q" (x) : "0" (x));
#else
		x = ((x & 0x00ff) << 8) | ((x & 0xff00) >> 8);
#endif
	}
	return x;
}
#define htods dtohs
/* host <--> disk for "long"s */
static inline unsigned long dtohl (struct ufs_sb_info * sb, unsigned long x)
{
	if (sb->fs_convert) {
#ifdef __i386__
		__asm__("xchgb %b0,%h0 ; rorl $16,%0 ; xchgb %b0,%h0"
			: "=q" (x) : "0" (x));
#else
		x = ((x & 0x000000ff) << 24) | ((x & 0x0000ff00) << 8)
		    | ((x & 0x00ff0000) >> 8) | ((x & 0xff000000) >> 24);
#endif
	}
	return x;
}
#define htodl dtohl
/* host <--> disk for "long long"s ("quad"s) */
static inline unsigned long long dtohq (struct ufs_sb_info * sb, unsigned long long x)
{
	if (sb->fs_convert) {
		unsigned long x0 = (unsigned long) x;
		unsigned long x1 = (unsigned long) (x >> 32);
#ifdef __i386__
		__asm__("xchgb %b0,%h0 ; rorl $16,%0 ; xchgb %b0,%h0"
			: "=q" (x0) : "0" (x0));
		__asm__("xchgb %b0,%h0 ; rorl $16,%0 ; xchgb %b0,%h0"
			: "=q" (x1) : "0" (x1));
#else
		x0 = ((x0 & 0x000000ff) << 24) | ((x0 & 0x0000ff00) << 8)
		    | ((x0 & 0x00ff0000) >> 8) | ((x0 & 0xff000000) >> 24);
		x1 = ((x1 & 0x000000ff) << 24) | ((x1 & 0x0000ff00) << 8)
		    | ((x1 & 0x00ff0000) >> 8) | ((x1 & 0xff000000) >> 24);
#endif
		x = ((unsigned long long) x0 << 32) | (unsigned long long) x1;
	}
	return x;
}
#define htodq dtohq

extern struct inode_operations ufs_file_inode_operations;
extern struct inode_operations ufs_dir_inode_operations;
extern struct inode_operations ufs_symlink_inode_operations;

extern int ufs_lookup(struct inode * dir,const char * name, int len,
        	      struct inode ** result);
extern int ufs_readdir(struct inode *inode, struct file *filp,
	        struct dirent *dirent, int count);
extern struct super_block *ufs_read_super(struct super_block *s,void *data,
				     int silent);
extern void ufs_read_inode(struct inode *inode);
extern struct buffer_head *ufs_getblk(struct inode *inode, int bn);
extern struct buffer_head *ufs_bread(struct inode *inode, int bn);
extern daddr_t ufs_blknum(struct inode *inode, int bn);

#endif
